@@ -32,11 +32,7 @@ import platform
 import re
 import sys
 from site import USER_BASE
-
-# also considered colorama and crayons
-# 1. colorama requires to send reset codes. annoying.
-# 2. crayons is a wrapper around colorama, not in debian
-import termcolor
+from typing import Iterable, Optional, Sequence
 
 # XXX: we *also* need pytz even though dateutil also ships time zone
 # info. pytz has the *list* of all time zones, which dateutil doesn't
@@ -60,6 +56,19 @@ import pytz
 # colors, see https://gitlab.com/anarcat/undertime/issues/9 and
 # https://github.com/Robpol86/terminaltables/issues/55
 import tabulate
+
+# also considered colorama and crayons
+# 1. colorama requires to send reset codes. annoying.
+# 2. crayons is a wrapper around colorama, not in debian
+import termcolor
+
+# also considered dateutil.tz.tzlocal() but that returns a rather
+# opaque tzinfo object that I haven't figured out how to use. by
+# contrast, tzlocal returns a more modern pytz object that actually
+# has a proper underlying tzinfo object. it's possible to extract a
+# timezone name out of dateutil, but it first needs a timestamp and we
+# can't do that because we need the zone to parse the time.
+import tzlocal
 import yaml
 from dateutil.relativedelta import relativedelta
 
@@ -119,12 +128,12 @@ class ConfigAction(argparse._StoreAction):
     to fix that are welcome.
     """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs):  # type: ignore[no-untyped-def]
         """the config action is a search path, so a list, so one or more argument"""
         kwargs["nargs"] = 1
         super().__init__(*args, **kwargs)
 
-    def __call__(self, parser, ns, values, option):
+    def __call__(self, parser, ns, values, option):  # type: ignore[no-untyped-def]
         """change defaults for the namespace, still allows overriding
         from commandline options"""
         for path in values:
@@ -140,15 +149,15 @@ class ConfigAction(argparse._StoreAction):
                 break
         super().__call__(parser, ns, values, option)
 
-    def parse_config(self, path):
-        """abstract implementation of config file parsing, should be overriden in subclasses"""
+    def parse_config(self, path: str) -> dict:  # type: ignore[type-arg]
+        """abstract implementation of config file parsing, should be overridden in subclasses"""
         raise NotImplementedError()
 
 
 class YamlConfigAction(ConfigAction):
     """YAML config file parser action"""
 
-    def parse_config(self, path):
+    def parse_config(self, path: str) -> dict:  # type: ignore[type-arg]
         """This doesn't handle errors around open() and others, callers should
         probably catch FileNotFoundError at least.
         """
@@ -158,7 +167,7 @@ class YamlConfigAction(ConfigAction):
                 return yaml.safe_load(handle) or {}
         except yaml.error.YAMLError as e:
             raise argparse.ArgumentError(
-                self, "failed to parse YAML configuration: %s" % e
+                self, "failed to parse YAML configuration: %s" % str(e)
             )
 
 
@@ -241,15 +250,26 @@ class ConfigArgumentParser(argparse.ArgumentParser):
     'argh'
     >>> c.close()
     >>> d.close()
+
+    There are actually many other implementations of this we might
+    want to consider instead of maintaining our own:
+
+    https://github.com/omni-us/jsonargparse
+    https://github.com/bw2/ConfigArgParse
+    https://github.com/omry/omegaconf
+
+    See this comment for a quick review:
+
+    https://github.com/borgbackup/borg/issues/6551#issuecomment-1094104453
     """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs):  # type: ignore[no-untyped-def]
         super().__init__(*args, **kwargs)
         # a list of actions to fire with their defaults if not fired
         # during parsing
         self._delayed_config_action = []
 
-    def _add_action(self, action):
+    def _add_action(self, action: argparse.Action) -> argparse.Action:  # type: ignore[override]
         # this overrides the add_argument() routine, which is where
         # actions get registered in the argparse module.
         #
@@ -264,8 +284,13 @@ class ConfigArgumentParser(argparse.ArgumentParser):
         action = super()._add_action(action)
         if isinstance(action, ConfigAction) and action.default is not None:
             self._delayed_config_action.append(action)
+        return action
 
-    def parse_args(self, args=None, namespace=None):
+    def parse_args(  # type: ignore[override]
+        self,
+        args: Optional[Sequence[str]] = None,
+        namespace: Optional[argparse.Namespace] = None,
+    ) -> argparse.Namespace:
         # we do a first failsafe pass on the commandline to find out
         # if we have any "config" parameters specified, in which case
         # we must *not* load the default config file
@@ -288,12 +313,13 @@ class ConfigArgumentParser(argparse.ArgumentParser):
                 continue
             logging.debug("searching config files: %s" % action.default)
             action(self, ns, action.default, None)
+        # this will actually load the relevant config file when found
         # on the commandline
         #
         # note that this will load the config file a second time
         return super().parse_args(args, namespace)
 
-    def default_config(self):
+    def default_config(self) -> Iterable[str]:
         """handy shortcut to detect commonly used config paths
 
         This list is processed as a FIFO: if a file is found in there,
@@ -303,7 +329,7 @@ class ConfigArgumentParser(argparse.ArgumentParser):
             os.path.join(
                 os.environ.get("XDG_CONFIG_HOME", "~/.config/"), self.prog + ".yml"
             ),
-            os.path.join(USER_BASE, "etc", self.prog + ".yml"),
+            os.path.join(USER_BASE or "/usr/local", "etc", self.prog + ".yml"),
             os.path.join("/usr/local/etc", self.prog + ".yml"),
             os.path.join("/etc", self.prog + ".yml"),
         ]
@@ -312,11 +338,11 @@ class ConfigArgumentParser(argparse.ArgumentParser):
 class LoggingAction(argparse.Action):
     """change log level on the fly
 
-    The logging system should be initialized befure this, using
+    The logging system should be initialized before this, using
     `basicConfig`.
     """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs):  # type: ignore[no-untyped-def]
         """setup the action parameters
 
         This enforces a selection of logging levels. It also checks if
@@ -328,12 +354,14 @@ class LoggingAction(argparse.Action):
             kwargs["nargs"] = 0
         super().__init__(*args, **kwargs)
 
-    def __call__(self, parser, ns, values, option):
+    def __call__(self, parser, ns, values, option):  # type: ignore[no-untyped-def]
         """if const was specified it means argument-less parameters"""
         if self.const:
             logging.getLogger("").setLevel(self.const)
         else:
             logging.getLogger("").setLevel(values)
+        # cargo-culted from _StoreConstAction
+        setattr(ns, self.dest, self.const or values)
 
 
 class UndertimeArgumentParser(ConfigArgumentParser):
@@ -344,7 +372,8 @@ class UndertimeArgumentParser(ConfigArgumentParser):
             description="pick a meeting time", epilog=__doc__, *args, **kwargs
         )
         # do not forget to change the manpage and shell completion in
-        # extra/ when you change anything below
+        # extra/ when you change anything below, consider
+        # https://github.com/iterative/shtab
         group = self.add_argument_group("time and date options")
         group.add_argument(
             "-d",
@@ -495,9 +524,6 @@ class UndertimeArgumentParser(ConfigArgumentParser):
             action="store_true",
             help="show valid time zones and exit",
         )
-        group.add_argument(
-            "--selftest", action="store_true", help="run test suite and exit"
-        )
         group.add_argument(
             "-V",
             "--version",
@@ -534,44 +560,6 @@ class OffsetZone(pytz._FixedOffset):
     This asserts the string is like UTC+X or UTC-X (see the `regex`
     below for the exact pattern). It will also raise a ValueError for
     invalid offsets.
-
-    >>> OffsetZone("UTC+2")._minutes // 60
-    2
-    >>> OffsetZone("GMT-4")._minutes // 60
-    -4
-    >>> OffsetZone("UTC+3:30")._minutes
-    210
-    >>> OffsetZone("UTC+00:30")._minutes
-    30
-    >>> OffsetZone("UTC-00:30")._minutes
-    -30
-    >>> OffsetZone("UTC-01:30")._minutes
-    -90
-    >>> OffsetZone("UTC-13")
-    Traceback (most recent call last):
-        ...
-    ValueError: Hours cannot be bigger than 12: 13
-    >>> OffsetZone("GMT+20")
-    Traceback (most recent call last):
-        ...
-    ValueError: Hours cannot be bigger than 12: 20
-    >>> OffsetZone("America/Eastern")
-    Traceback (most recent call last):
-        ...
-    AssertionError
-    >>> OffsetZone("UTC+3:70")._minutes // 60
-    Traceback (most recent call last):
-        ...
-    ValueError: Minute cannot be bigger than 59: 70
-    >>> OffsetZone("UTC+3,30")
-    Traceback (most recent call last):
-        ...
-    AssertionError
-    >>> OffsetZone("GMT-3.14159")
-    Traceback (most recent call last):
-        ...
-    AssertionError
-
     """
 
     regex = re.compile(r"^(?:UTC|GMT)(?P<sign>[-+])(?P<hours>\d+)(:(?P<minutes>\d+))?$")
@@ -662,27 +650,26 @@ def parse_date(date, local_zone):
     A correctly parsed date should be returned as a datetime object.
 
     When a new parser is added here, make sure to also report its
-    version in the --selftest output, currently in sysinfo().
+    version in sysinfo().
 
     Note that all date parsers fail in some way, each function details
     the known failures.
-
     """
     now = None
     if date is None:
         return datetime.datetime.now(local_zone)
-    logging.debug("trying to parse date with dateparser module")
+    logging.debug("trying to parse date '%s' with dateparser module", date)
     now = parse_date_dateparser(date, local_zone)
     if now:
         return now
-    logging.debug("trying to parse date with parsedatetime module")
+    logging.debug("trying to parse date '%s' with parsedatetime module", date)
     now = parse_date_parsedatetime(date, local_zone)
     if now:
         return now
-    logging.debug("trying to parse date with arrow module")
+    logging.debug("trying to parse date '%s' with arrow module", date)
     now = parse_date_arrow(date)
     if not now:
-        logging.error("date provided cannot be parsed: %s", date)
+        logging.error("date provided cannot be parsed: '%s'", date)
     return now
 
 
@@ -792,115 +779,10 @@ def flush_logging_handlers():
 def main(args):
     """Main entry point.
 
-    Tests for the two corner cases in US/Eastern in 2020. We don't
+    Tests for the two corner cases in America/New_York in 2020. We don't
     really want to test *all* of those corner cases here, but the
     first one of those caused me problems at that time and I wanted to
-    have a good handle on it.
-
-    #doctest:ELLIPSIS
-    >>> argv = ['--config', '/dev/null', '--no-default', '--no-unique']
-    >>> date = ['--date', "2020-03-08 12:00:00-04:00"]
-    >>> zones = ['--timezone', 'EST', 'US/Eastern', 'UTC']
-    >>> args = UndertimeArgumentParser().parse_args(argv + date + zones)
-    >>> main(args)
-    ╔════════╤══════════════╤════════╤═══════════╗
-    ║  EST   │  US/Eastern  │  UTC   │   overlap ║
-    ╠════════╪══════════════╪════════╪═══════════╣
-    ║ 19:00  │    19:00     │ 00:00  │         0 ║
-    ║ 20:00  │    20:00     │ 01:00  │         0 ║
-    ║ 21:00  │    21:00     │ 02:00  │         0 ║
-    ║ 22:00  │    22:00     │ 03:00  │         0 ║
-    ║ 23:00  │    23:00     │ 04:00  │         0 ║
-    ║ 00:00  │    00:00     │ 05:00  │         0 ║
-    ║ 01:00  │    01:00     │ 06:00  │         0 ║
-    ║ 02:00  │    03:00     │ 07:00  │         0 ║
-    ║ 03:00  │    04:00     │ 08:00  │         0 ║
-    ║ 04:00  │    05:00     │ 09:00_ │         1 ║
-    ...
-    local time requested: 2020-03-08 16:00:00+00:00
-    equivalent to: 11:00 EST, 12:00 US/Eastern, 16:00 UTC
-    >>> date = ['--date', '2020-11-01 12:00:00-04:00']
-    >>> args = UndertimeArgumentParser().parse_args(argv + date + zones)
-    >>> main(args)
-    ╔════════╤══════════════╤════════╤═══════════╗
-    ║  EST   │  US/Eastern  │  UTC   │   overlap ║
-    ╠════════╪══════════════╪════════╪═══════════╣
-    ║ 19:00  │    20:00     │ 00:00  │         0 ║
-    ║ 20:00  │    21:00     │ 01:00  │         0 ║
-    ║ 21:00  │    22:00     │ 02:00  │         0 ║
-    ║ 22:00  │    23:00     │ 03:00  │         0 ║
-    ║ 23:00  │    00:00     │ 04:00  │         0 ║
-    ║ 00:00  │    01:00     │ 05:00  │         0 ║
-    ║ 01:00  │    01:00     │ 06:00  │         0 ║
-    ║ 02:00  │    02:00     │ 07:00  │         0 ║
-    ║ 03:00  │    03:00     │ 08:00  │         0 ║
-    ║ 04:00  │    04:00     │ 09:00_ │         1 ║
-    ...
-    local time requested: 2020-11-01 16:00:00+00:00
-    equivalent to: 11:00 EST, 11:00 US/Eastern, 16:00 UTC
-    >>> extra = [ '--no-overlap', '--truncate' ]
-    >>> args = UndertimeArgumentParser().parse_args(argv + date + zones + extra)
-    >>> main(args)
-    ╔════════╤═══════════╤════════╗
-    ║  EST   │  Eastern  │  UTC   ║
-    ╠════════╪═══════════╪════════╣
-    ║ 19:00  │   20:00   │ 00:00  ║
-    ...
-    >>> extra = [ '--overlap', '--truncate' ]
-    >>> args = UndertimeArgumentParser().parse_args(argv + date + zones + extra)
-    >>> main(args)
-    ╔════════╤═══════════╤════════╤═════╗
-    ║  EST   │  Eastern  │  UTC   │   n ║
-    ╠════════╪═══════════╪════════╪═════╣
-    ║ 19:00  │   20:00   │ 00:00  │   0 ║
-    ...
-    >>> extra = [ '--no-table' ]
-    >>> args = UndertimeArgumentParser().parse_args(argv + date + zones + extra)
-    >>> main(args)
-    UTC time requested: 2020-11-01 16:00:00+00:00
-    local time requested: 2020-11-01 16:00:00+00:00
-    equivalent to: 11:00 EST, 11:00 US/Eastern, 16:00 UTC
-    >>> zones = ['--timezone', 'EST', 'US/Eastern', 'UTC', 'Pacific/Auckland']
-    >>> args = UndertimeArgumentParser().parse_args(argv + date + zones + extra)
-    >>> main(args)
-    UTC time requested: 2020-11-01 16:00:00+00:00
-    local time requested: 2020-11-01 16:00:00+00:00
-    equivalent to: 11:00 EST, 11:00 US/Eastern, 16:00 UTC, 2020-11-02 05:00 Pacific/Auckland
-    >>> logging.getLogger('').setLevel('WARNING')
-    >>> argv = ['--config', '/dev/null', '--no-default']
-    >>> args = UndertimeArgumentParser().parse_args(argv + date + [ '--timezones', 'CEST' ])
-    >>> main(args)
-    Traceback (most recent call last):
-        ...
-    SystemExit: 1
-    >>> logging.getLogger('').setLevel('WARNING')
-    >>> argv = ['--config', '/dev/null', '--no-default']
-    >>> args = UndertimeArgumentParser().parse_args(argv)
-    >>> main(args)
-    Traceback (most recent call last):
-        ...
-    SystemExit: 1
-    >>> argv = ['--config', '/dev/null', '--no-default']
-    >>> args = UndertimeArgumentParser().parse_args(argv + ["NOTADATE"])
-    >>> main(args)
-    Traceback (most recent call last):
-        ...
-    SystemExit: 1
-    >>> msgs = flush_logging_handlers()
-    >>> assert 'unknown zone, skipping: CEST' in msgs
-    >>> assert 'No valid time zone found.' in msgs
-    >>> args = UndertimeArgumentParser().parse_args(argv + ['--list-zones'])
-    >>> main(args)
-    Africa/Abidjan
-    ...
-    Zulu
-    >>> args = UndertimeArgumentParser().parse_args(argv + ['--version'])
-    Traceback (most recent call last):
-        ...
-    SystemExit: 0
-    >>> args = UndertimeArgumentParser().parse_args(argv + ["2020-03-08", "12:00:00-04:00"])
-    >>>
-    """
+    have a good handle on it."""
     if args.list_zones:
         print("\n".join(pytz.all_timezones))
         return
@@ -910,9 +792,10 @@ def main(args):
         fmt_time = fmt_time_ascii
 
     # find the local timezone
-    now_zone = get_localzone()
+    default_zone = tzlocal.get_localzone()
+    logging.debug("local zone: %s", default_zone)
     # make an educated guess at what the user meant by passing that time zone to parse_date
-    then_local = parse_date(args.date, now_zone)
+    then_local = parse_date(args.date, default_zone)
     if not then_local:
         sys.exit(1)
     # round off microseconds to cleanup display, in case we use "now"
@@ -922,12 +805,10 @@ def main(args):
     then_local = then_local.replace(microsecond=0)
     # convert that time to UTC again
     then_utc = then_local.astimezone(datetime.timezone.utc)
-    # and guess what time zone that was *then*
-    then_zone = get_localzone(then_utc)
 
     timezones = []
     if args.default_zone:
-        timezones.append(then_zone)
+        timezones.append(default_zone)
     timezones += filter(None, [guess_zone(z) for z in args.timezones])
     if args.unique:
         timezones = list(uniq_zones(timezones, then_utc))
@@ -970,7 +851,7 @@ def main(args):
         if local_zone_day != other_zone_day:
             logging.debug(
                 "different day: local (%s) is %s other (%s) is %s",
-                then_zone,
+                default_zone,
                 local_zone_day,
                 zone,
                 other_zone_day,
@@ -979,53 +860,11 @@ def main(args):
         else:
             ts = "{0:%H:%M} {1}".format(then_local.astimezone(tz=zone).timetz(), zone)
         times.append(ts)
-    print("UTC time requested: {}".format(then_utc))
-    print("local time requested: {}".format(then_local))
+    print("UTC time: {}".format(then_utc))
+    print("local time: {}".format(then_local))
     print("equivalent to: " + ", ".join(times))
 
 
-def get_localzone(now_utc=None):
-    """find the time zone on the local system
-
-    This was mostly cargo-culted from:
-
-    https://stackoverflow.com/a/39079819/1174784
-
-    ... but it also happens to work. To avoid such cargo-culting, we'd
-    need a entire module and I'm unwilling to add a dependency just to
-    remove this single function:
-
-    https://github.com/regebro/tzlocal
-
-    This is still a "tricky use" of datetime's code (one would say a
-    horrible hack), which, arguably, should really just be part of the
-    datetime module itself.
-
-    This unit test will succeed only if TZ=UTC is set in the
-    environment (which it is, in tox):
-
-    >>> n = datetime.datetime(2019, 5, 18, 15, 17, tzinfo=datetime.timezone.utc)
-    >>> str(get_localzone(n))
-    'UTC'
-
-    It's also kind of dumb because it somewhat proves nothing: it
-    doesn't show that it can find other timezones. But doing so would
-    require monkeypatching that environment, and I'm not quite ready
-    to mess with that yet. And besides, this comment is now about a
-    hundred times longer than the actual frigging code so I better
-    stop writing.
-    """
-    if now_utc is None:
-        # first we get a timestamp with the the UTC timezone
-        now_utc = datetime.datetime.now(datetime.timezone.utc)
-    # then, mysteriously, getting the timezone info back from *that*
-    # object gives us the *local* timezone. go figure. it's probably
-    # because we do not pass an argument to `astimezone` so it does
-    # our work of finding the right timezone, but really, that's just
-    # a guess at this point.
-    return now_utc.astimezone().tzinfo
-
-
 def guess_zone(zone):
     """
     guess a zone from a string, based on pytz
@@ -1075,7 +914,7 @@ def uniq_zones(timezones, now):
 
     >>> local_zone = datetime.timezone(datetime.timedelta(days=-1, seconds=72000), 'EDT')
     >>> now = parse_date('2020-03-08 22:30', local_zone=local_zone)
-    >>> zones = [guess_zone('Toronto'), guess_zone('Canada/Eastern')]
+    >>> zones = [guess_zone('Toronto'), guess_zone('New_York')]
     >>> list(uniq_zones(zones, now))
     [<DstTzInfo 'America/Toronto' LMT-1 day, 18:42:00 STD>]
     """
@@ -1100,25 +939,6 @@ def uniq_zones(timezones, now):
 def compute_table(
     now_local, timezones, start, end, overlap_min=0, overlap_show=True, truncate=False
 ):
-    """
-    >>> local_zone = datetime.timezone(datetime.timedelta(days=-1, seconds=72000), 'EDT')
-    >>> now = parse_date('2020-03-08 22:30', local_zone=local_zone)
-    >>> nearest_hour = now.replace(minute=0, second=0, microsecond=0)
-    >>> nearest_hour
-    datetime.datetime(2020, 3, 8, 22, 0, tzinfo=<StaticTzInfo 'EDT'>)
-    >>> start_time = current_time = nearest_hour.replace(hour=0)
-    >>> start_time
-    datetime.datetime(2020, 3, 8, 0, 0, tzinfo=<StaticTzInfo 'EDT'>)
-    >>> timezones = []
-    >>> timezones.append(local_zone)
-    >>> timezones += [guess_zone(z) for z in ('US/Eastern', 'UTC')]
-    >>> [str(t) for t in timezones]
-    ['EDT', 'US/Eastern', 'UTC']
-    >>> [cell[:5] for row in compute_table(now, timezones, 9, 17)[1:5] for cell in row]
-    ['00:00', '23:00', '04:00', '0', '01:00', '00:00', '05:00', '0', '02:00', '01:00', '06:00', '0', '03:00', '03:00', '07:00', '0']
-    >>> [cell[:5] for row in compute_table(now, timezones, 9, 17, 0, False)[1:5] for cell in row]
-    ['00:00', '23:00', '04:00', '01:00', '00:00', '05:00', '02:00', '01:00', '06:00', '03:00', '03:00', '07:00']
-    """  # noqa: E501
     # compute the earlier local midnight
     nearest_midnight = now_local + relativedelta(
         hour=0, minute=0, seconds=0, microseconds=0
@@ -1176,8 +996,12 @@ def compute_table(
     return rows
 
 
-def sysinfo(callback=logging.info):  # pragma: nocover
-    """dump a lot of system information to help with support"""
+def sysinfo(callback=logging.debug):  # pragma: nocover
+    """dump a lot of system information to help with support
+
+    A more elaborate and possibly more complete implementation of this
+    is https://github.com/banesullivan/scooby
+    """
     callback("file: %s", __file__)
     callback("command: %r", sys.argv)
     callback("version: %s", ImportlibVersionAction._version())
@@ -1244,37 +1068,25 @@ def sysinfo(callback=logging.info):  # pragma: nocover
 
     callback("timezone database version: %s", pytz.OLSON_VERSION)
 
+
 if __name__ == "__main__":  # pragma: nocover
     logging.basicConfig(format="%(levelname)s: %(message)s", level="WARNING")
     parser = UndertimeArgumentParser()
     args = parser.parse_args()
-    if args.selftest:
 
-        def print_as_log(format, *args):
-            print(format % args)
+    sysinfo(callback=logging.debug)
+    try:
+        main(args)
+    except Exception as e:
+        logging.error("unexpected error: %s", e)
+        if args.debug:
+            logging.warning(
+                'starting debugger, type "c" and enter to continue'
+            )  # noqa: E501
+            import pdb
+            import traceback
 
-        sysinfo(callback=print_as_log)
-        # reset loggers
-        logging.getLogger().handlers = []
-        # log to memory so that doctest can check output and we do not
-        # pollute --selftest.
-        #
-        # XXX: it is not clear to me why we need to do this in
-        # Doctest. I would expect doctest to just capture the logging
-        # output as any other, but it seems to treat print()
-        # differently than logging. so instead of stdout or stderr
-        # stream handlers here, we need to use a BufferingHandler and
-        # manually check it.
-        logging.getLogger().addHandler(logging.handlers.BufferingHandler(10))
-        # tests are designed to run with a UTC time zone
-        os.environ["TZ"] = "UTC"
-        import doctest
-
-        result = doctest.testmod(optionflags=doctest.ELLIPSIS)
-        if result.failed:
-            print("test suite failed")
-        else:
-            print("test suite succeeded")
-        sys.exit(result.failed)
-
-    main(args)
+            traceback.print_exc()
+            pdb.post_mortem()
+            sys.exit(1)
+        raise e
